/*
 * Copyright 2015-present Open Networking Foundation
 * Copyright © 2020 camunda services GmbH (info@camunda.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.atomix.raft.storage.log.entry;

import static com.google.common.base.MoreObjects.toStringHelper;

import io.atomix.utils.misc.TimestampPrinter;
import java.util.Objects;

/** Base class for timestamped entries. */
public abstract class TimestampedEntry extends RaftLogEntry {

  protected final long timestamp;

  public TimestampedEntry(final long term, final long timestamp) {
    super(term);
    this.timestamp = timestamp;
  }

  /**
   * Returns the entry timestamp.
   *
   * @return The entry timestamp.
   */
  public long timestamp() {
    return timestamp;
  }

  @Override
  public String toString() {
    return toStringHelper(this)
        .add("term", term)
        .add("timestamp", new TimestampPrinter(timestamp))
        .toString();
  }

  @Override
  public int hashCode() {
    return Objects.hash(timestamp);
  }

  @Override
  public boolean equals(final Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    final TimestampedEntry that = (TimestampedEntry) o;
    return timestamp == that.timestamp;
  }
}
